package com.qmth.sop.business.service.impl;

import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.dto.DataPermissionDto;
import com.qmth.sop.business.bean.dto.DeviceDeliveryImportDto;
import com.qmth.sop.business.bean.dto.DeviceInfoDto;
import com.qmth.sop.business.bean.params.DeviceDeliveryParam;
import com.qmth.sop.business.bean.result.TBDeviceDeliveryResult;
import com.qmth.sop.business.entity.SysDevice;
import com.qmth.sop.business.entity.SysSupplier;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBDeviceDelivery;
import com.qmth.sop.business.mapper.TBDeviceDeliveryMapper;
import com.qmth.sop.business.service.*;
import com.qmth.sop.business.util.excel.BasicExcelListener;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.*;
import com.qmth.sop.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 设备发货表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-11-28
 */
@Service
public class TBDeviceDeliveryServiceImpl extends ServiceImpl<TBDeviceDeliveryMapper, TBDeviceDelivery> implements TBDeviceDeliveryService {

    @Resource
    private SysDeviceService sysDeviceService;
    @Resource
    private SysDeviceModelService sysDeviceModelService;
    @Resource
    private SysSupplierService sysSupplierService;
    @Resource
    private SysUserService sysUserService;
    @Resource
    private BasicAttachmentService basicAttachmentService;

    @Transactional
    @Override
    public void deviceDeliveryImport(MultipartFile file) throws IOException {
        InputStream inputStream = file.getInputStream();
        LinkedMultiValueMap<String, DeviceDeliveryImportDto> deviceDeliveryImportDtoLinkedMultiValueMap = new LinkedMultiValueMap<>();

        StringJoiner errorData = new StringJoiner("");
        EasyExcel.read(inputStream, DeviceDeliveryImportDto.class, new BasicExcelListener<DeviceDeliveryImportDto>() {
            @Override
            public void handle(LinkedMultiValueMap<String, DeviceDeliveryImportDto> dataList, StringJoiner errorDataSj, Exception exception) {
                deviceDeliveryImportDtoLinkedMultiValueMap.addAll(dataList);
                errorData.add(errorDataSj.toString());
            }
        }).headRowNumber(2).sheet(0).doRead();

        List<DeviceDeliveryImportDto> deviceDeliveryImportDtoList = deviceDeliveryImportDtoLinkedMultiValueMap.get(BasicExcelListener.SUCCESS);
        if (CollectionUtils.isNotEmpty(deviceDeliveryImportDtoList)) {
            List<SysSupplier> sysSupplierList = sysSupplierService.list(new QueryWrapper<SysSupplier>().lambda().eq(SysSupplier::getType, SupplierTypeEnum.DEVICE));
            // 供应商map
            Map<String, Long> supplierMap = sysSupplierList.stream().collect(Collectors.toMap(SysSupplier::getName, SysSupplier::getId));

            // serialNo - obj
            Map<String, DeviceInfoDto> deviceMap = new HashMap<>();
            for (DeviceDeliveryImportDto deviceDeliveryImportDto : deviceDeliveryImportDtoList) {
                String serialNo = deviceDeliveryImportDto.getSerialNo();
                String deviceNo = deviceDeliveryImportDto.getDeviceNo();
                // 品牌
                String brand = deviceDeliveryImportDto.getBrand();
                // 型号
                String model = deviceDeliveryImportDto.getModel();
                if (deviceMap.containsKey(serialNo)) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("excel中存在重复的设备序列号[%s]", serialNo));
                } else {
                    DeviceInfoDto deviceInfo = new DeviceInfoDto();
                    deviceInfo.setSerialNo(serialNo);
                    deviceInfo.setDeviceNo(deviceNo);
                    deviceInfo.setBrand(brand);
                    deviceInfo.setModel(model);
                    deviceMap.put(serialNo, deviceInfo);
                }
                String crmNo = deviceDeliveryImportDto.getCrmNo();

                String supplierName = deviceDeliveryImportDto.getSupplierName();
                Long supplierId = supplierMap.get(supplierName);
                if (supplierId == null || supplierId == 0) {
                    throw ExceptionResultEnum.ERROR.exception(String.format("excel中存在系统中不存在的供应商名称[%s]", supplierName));
                }

                String mailingAddress = deviceDeliveryImportDto.getMailingAddress();
                String consignee = deviceDeliveryImportDto.getConsignee();
                String consigneePhone = deviceDeliveryImportDto.getConsigneePhone();
                String expressNo = deviceDeliveryImportDto.getExpressNo();

                // 查询设备表是否有该设备
                SysDevice sysDevice = sysDeviceService.getOne(new QueryWrapper<SysDevice>()
                        .lambda()
                        .eq(SysDevice::getSerialNo, serialNo)
                        .last(SystemConstant.LIMIT1));

                if (Objects.isNull(sysDevice)) {
                    // 设备表不存在该设备 -> 新增设备
                    sysDeviceModelService.findOrAddDeviceModel(brand, model);

                    SysDevice willAddSysDevice = new SysDevice();
                    willAddSysDevice.setDeviceCode(deviceNo);
                    willAddSysDevice.setSerialNo(serialNo);
                    willAddSysDevice.setBrand(brand);
                    willAddSysDevice.setModel(model);
                    willAddSysDevice.setBuyTime(System.currentTimeMillis());
                    willAddSysDevice.setSupplierId(supplierId);
                    willAddSysDevice.setStatus(DeviceStatusEnum.NORMAL);
                    willAddSysDevice.setLocation(supplierName);
                    willAddSysDevice.setBound(InOutTypeEnum.IN);
                    willAddSysDevice.setEnable(true);
                    sysDeviceService.save(willAddSysDevice);
                }

                // 查询是否有发货设备
                TBDeviceDelivery dbDeviceDelivery = this.getOne(new QueryWrapper<TBDeviceDelivery>()
                        .lambda()
                        .eq(TBDeviceDelivery::getSerialNo, serialNo)
                        .eq(TBDeviceDelivery::getEffect, true)
                        .last(SystemConstant.LIMIT1));

                if (Objects.nonNull(dbDeviceDelivery)) {
                    throw ExceptionResultEnum.EXCEPTION_ERROR.exception(String.format("excel中存在已发货的设备(序列号)[%s]", serialNo));
                }

                TBDeviceDelivery tbDeviceDelivery = new TBDeviceDelivery();
                if (crmNo != null && crmNo.length() > 0) {
                    tbDeviceDelivery.setCrmNo(crmNo);
                    tbDeviceDelivery.setUsageType(DeviceUsageTypeEnum.PROJECT);
                } else {
                    tbDeviceDelivery.setUsageType(DeviceUsageTypeEnum.OTHER);
                }
                tbDeviceDelivery.setSerialNo(serialNo);
                tbDeviceDelivery.setSupplierId(supplierId);
                tbDeviceDelivery.setMailingAddress(mailingAddress);
                tbDeviceDelivery.setConsignee(consignee);
                tbDeviceDelivery.setConsigneePhone(consigneePhone);
                tbDeviceDelivery.setExpressNo(expressNo);
                tbDeviceDelivery.setEnable(true);
                tbDeviceDelivery.setEffect(true);
                tbDeviceDelivery.setDeliveryType(InOutTypeEnum.OUT);
                tbDeviceDelivery.setStatus(DeviceDeliveryStatusEnum.UN_DELIVER);
                this.save(tbDeviceDelivery);
            }
        }
    }

    @Transactional
    @Override
    public void editDeviceDelivery(DeviceDeliveryParam deviceDeliveryParam) {
        Long id = deviceDeliveryParam.getId();
        String mailingAddress = deviceDeliveryParam.getMailingAddress();
        String consignee = deviceDeliveryParam.getConsignee();
        String consigneePhone = deviceDeliveryParam.getConsigneePhone();
        String expressNo = deviceDeliveryParam.getExpressNo();
        this.update(new UpdateWrapper<TBDeviceDelivery>().lambda()
                .eq(TBDeviceDelivery::getId, id)
                .set(TBDeviceDelivery::getMailingAddress, mailingAddress)
                .set(TBDeviceDelivery::getConsignee, consignee)
                .set(TBDeviceDelivery::getConsigneePhone, consigneePhone)
                .set(TBDeviceDelivery::getExpressNo, expressNo));
    }

    @Override
    public IPage<TBDeviceDeliveryResult> page(IPage<Map> iPage, Long serviceId, DeviceUsageTypeEnum usageType, String crmNo, DeviceDeliveryStatusEnum status, Long deliverUserId, Long deliveryStartTime, Long deliveryEndTime, String serialNo, Long supplierId, Long receiveStartTime, Long receiveEndTime) throws Exception {
        if (crmNo != null && crmNo.length() > 0) {
            crmNo = SystemConstant.translateSpecificSign(crmNo);
        }
        if (serialNo != null && serialNo.length() > 0) {
            serialNo = SystemConstant.translateSpecificSign(serialNo);
        }
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(requestUser.getId());
        if (dpr.getHasDeviceDelivery()) {
            // 是设备管理员
            Long userSupplierId = requestUser.getSupplierId();
            if (userSupplierId == null || userSupplierId == 0) {
                throw ExceptionResultEnum.ERROR.exception("请先在用户管理中绑定设备发货员对应的供应商");
            }
            supplierId = userSupplierId;
        }
        IPage<TBDeviceDeliveryResult> page = this.baseMapper.page(iPage, serviceId, usageType, crmNo, status, deliverUserId, deliveryStartTime, deliveryEndTime, serialNo, supplierId, receiveStartTime, receiveEndTime);
        for (TBDeviceDeliveryResult e : page.getRecords()) {
            this.fillResultInfo(e);
        }
        return page;
    }


    @Override
    public List<TBDeviceDeliveryResult> list(Long serviceId, DeviceUsageTypeEnum usageType, String crmNo, DeviceDeliveryStatusEnum status, Long deliverUserId, Long deliveryStartTime, Long deliveryEndTime, String serialNo, Long supplierId, Long receiveStartTime, Long receiveEndTime) throws Exception {
        if (crmNo != null && crmNo.length() > 0) {
            crmNo = SystemConstant.translateSpecificSign(crmNo);
        }
        if (serialNo != null && serialNo.length() > 0) {
            serialNo = SystemConstant.translateSpecificSign(serialNo);
        }
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionDto dpr = sysUserService.buildUserDataPermission(requestUser.getId());
        if (dpr.getHasDeviceDelivery()) {
            // 是设备管理员
            Long userSupplierId = requestUser.getSupplierId();
            if (userSupplierId == null || userSupplierId == 0) {
                throw ExceptionResultEnum.ERROR.exception("请先在用户管理中绑定设备发货员对应的供应商");
            }
            supplierId = userSupplierId;
        }
        List<TBDeviceDeliveryResult> result = this.baseMapper.list(serviceId, usageType, crmNo, status, deliverUserId, deliveryStartTime, deliveryEndTime, serialNo, supplierId, receiveStartTime, receiveEndTime);
        for (TBDeviceDeliveryResult e : result) {
            this.fillResultInfo(e);
        }
        return result;
    }

    /**
     * 补充结果信息
     *
     * @param e 发货结果对象
     * @throws Exception 异常
     */
    private void fillResultInfo(TBDeviceDeliveryResult e) throws Exception {
        DeviceUsageTypeEnum dbUsageType = e.getUsageType();
        e.setUsageTypeStr(Objects.nonNull(dbUsageType) ? dbUsageType.getTitle() : "");

        InOutTypeEnum dbDeliveryType = e.getDeliveryType();
        e.setDeliveryTypeStr(Objects.nonNull(dbDeliveryType) ? dbDeliveryType.getTitle() : "");

        DeviceDeliveryStatusEnum dbStatus = e.getStatus();
        e.setStatusStr(Objects.nonNull(dbStatus) ? dbStatus.getTitle() : "");

        Long dbDeliveryTime = e.getDeliverTime();
        e.setDeliverTimeStr(dbDeliveryTime != null && dbDeliveryTime > 0 ? DateFormatUtils.format(dbDeliveryTime, SystemConstant.DEFAULT_DATE_PATTERN) : "--");

        Long dbReceiveTime = e.getReceiveTime();
        e.setReceiveTimeStr(dbReceiveTime != null && dbReceiveTime > 0 ? DateFormatUtils.format(dbReceiveTime, SystemConstant.DEFAULT_DATE_PATTERN) : "--");

        String path = e.getExpressPhotoPath();
        if (path != null && path.length() > 0) {
            e.setExpressPhotoPath(basicAttachmentService.filePreviewPath(path));
        }
    }
}
